/*
  cadaverserver
  Copyright (C) 1999  Erik Thiele

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "conn.hpp"

#include "myspielfeld.hpp"
#include "misc.hpp"
#include "mycyborg.hpp"
#include "serverglobals.hpp"

#include <unistd.h>
#include <errno.h>
#include <stdio.h>
#include <sys/socket.h>
#include <math.h>

conn::~conn()
{
  for (map<const int, cyborg *const>::iterator i=gsc->all.begin();i!=gsc->all.end();i++)
    if (i->second->get_int(cyborg::OWNERID)==id())
      i->second->get_int(cyborg::OWNERID)=0;

  printf("player id=%d terminated\n",id());
  shutdown(handle,2);
  close(handle);
};

conn::conn(int handle_, struct sockaddr_in addr_)
  : bossmode(false), lastsex(false),
    stat_sentbytes(0), stat_recvbytes(0),
    handle(handle_), addr(addr_),
    oldfeld(gsf->xsize(),gsf->ysize()),
    visimap(gsf->xsize(),gsf->ysize())
{
  myid=getid();
  outputbuffer.reserve(100000);
  
  for (int x=0;x<oldfeld.xsize();x++)
    for (int y=0;y<oldfeld.ysize();y++)
      oldfeld.get(x,y).get_float(feld::WATER)=-2.0; // makes update neccessary

  sprintf(clip,"%d %d %d\n",gsf->xsize(),gsf->ysize(),id());
  outputbuffer+=clip;

  printf("new player connected, id=%d\n",id());
};

bool conn::manageinput ()
{
  int retval;

  while ((retval=read(handle,clip,1024))>0) {
    stat_recvbytes+=retval;
    clip[retval]=0;
    inputline+=clip;
  };
  if (retval==0) return false;
  else if (errno!=EAGAIN) return false;

  bool gotaQN=false;
  for (unsigned scan=0;scan<inputline.size();) {
    if (inputline[scan]<32) {
      if ((scan>=4)&&(inputline.substr(0,4)=="help")) {
	outputbuffer+="MM read the documentation in the server package!\n";
      }else if ((scan>=4)&&(inputline.substr(0,4)=="quit")) {
	return false;
      }else if ((scan>=2)&&(inputline.substr(0,2)=="QN")) {
	gotaQN=true;
      }else if ((scan>=3)&&(inputline.substr(0,3)=="TA ")) {
	int id,targettype;
	sscanf(inputline.substr(3,scan-3).c_str(),"%d %d",&id,&targettype);
	if (gsc->all.find(id)==gsc->all.end()) {
	  outputbuffer+="MM unexisting id in TA command\n";
	}else if (myid!=gsc->all[id]->get_int(cyborg::OWNERID)) {
	  outputbuffer+="MM am not owner\n";
	}else{
	  gsc->all[id]->get_int(cyborg::STATE)=cyborg::TAKE;
	  gsc->all[id]->get_int(cyborg::TAKEDROP)=targettype;
	};
      }else if ((scan>=3)&&(inputline.substr(0,3)=="DR ")) {
	int id,targettype;
	sscanf(inputline.substr(3,scan-3).c_str(),"%d %d",&id,&targettype);
	if (gsc->all.find(id)==gsc->all.end()) {
	  outputbuffer+="MM unexisting id in DR command\n";
	}else if (myid!=gsc->all[id]->get_int(cyborg::OWNERID)) {
	  outputbuffer+="MM am not owner\n";
	}else{
	  gsc->all[id]->get_int(cyborg::STATE)=cyborg::DROP;
	  gsc->all[id]->get_int(cyborg::TAKEDROP)=targettype;
	};
      }else if ((scan>=3)&&(inputline.substr(0,3)=="EA ")) {
	int id,targettype;
	sscanf(inputline.substr(3,scan-3).c_str(),"%d %d",&id,&targettype);
	if (gsc->all.find(id)==gsc->all.end()) {
	  outputbuffer+="MM unexisting id in EA command\n";
	}else if (myid!=gsc->all[id]->get_int(cyborg::OWNERID)) {
	  outputbuffer+="MM am not owner\n";
	}else{
	  gsc->all[id]->get_int(cyborg::STATE)=cyborg::EAT;
	  gsc->all[id]->get_int(cyborg::TAKEDROP)=targettype;
	};
      }else if ((scan>=3)&&(inputline.substr(0,3)=="CO ")) {
	int id,targettype;
	sscanf(inputline.substr(3,scan-3).c_str(),"%d %d",&id,&targettype);
	if (gsc->all.find(id)==gsc->all.end()) {
	  outputbuffer+="MM unexisting id in CO command\n";
	}else if (myid!=gsc->all[id]->get_int(cyborg::OWNERID)) {
	  outputbuffer+="MM am not owner\n";
	}else{
	  gsc->all[id]->get_int(cyborg::STATE)=cyborg::CONVERT;
	  gsc->all[id]->get_int(cyborg::TAKEDROP)=targettype;
	};
      }else if ((scan>=3)&&(inputline.substr(0,3)=="BU ")) {
	int id,targettype;
	sscanf(inputline.substr(3,scan-3).c_str(),"%d %d",&id,&targettype);
	if (gsc->all.find(id)==gsc->all.end()) {
	  outputbuffer+="MM unexisting id in BU command\n";
	}else if (myid!=gsc->all[id]->get_int(cyborg::OWNERID)) {
	  outputbuffer+="MM am not owner\n";
	}else{
	  gsc->all[id]->get_int(cyborg::STATE)=cyborg::BURN;
	  gsc->all[id]->get_int(cyborg::TAKEDROP)=targettype;
	};
      }else if ((scan>=3)&&(inputline.substr(0,3)=="MS ")) {
	int id,targetid;
	sscanf(inputline.substr(3,scan-3).c_str(),"%d %d",&id,&targetid);
	if (gsc->all.find(id)==gsc->all.end()) {
	  outputbuffer+="MM unexisting id in MS command\n";
	}else if (gsc->all.find(targetid)==gsc->all.end()) {
	  outputbuffer+="MM unexisting targetid in MS command\n";
	}else if (myid!=gsc->all[id]->get_int(cyborg::OWNERID)) {
	  outputbuffer+="MM am not owner\n";
	}else if (gsc->all[id]->get_bool(cyborg::ISMALE)==false) {
	  outputbuffer+="MM only male can start sex\n";
	}else if (gsc->all[targetid]->get_bool(cyborg::ISMALE)==true) {
	  outputbuffer+="MM target is not female\n";
	}else if (gsc->all[id]->get_float(cyborg::FERTILITY)>0.1) {
	  outputbuffer+="MM male not ready\n";
	}else if (gsc->all[targetid]->get_float(cyborg::FERTILITY)>0.1) {
	  outputbuffer+="MM female not ready\n";
	}else{
	  float myx=gsc->all[id]->get_float(cyborg::POSX);
	  float myy=gsc->all[id]->get_float(cyborg::POSY);
	  float itx=gsc->all[targetid]->get_float(cyborg::POSX);
	  float ity=gsc->all[targetid]->get_float(cyborg::POSY);
	  if (hypotf(myx-itx,myy-ity)>2.0) {
	    outputbuffer+="MM too far away for sex\n";
	  }else{
	    gsc->all[id]->get_float(cyborg::FERTILITY)=1.0;
	    gsc->all[targetid]->get_float(cyborg::FERTILITY)=1.0;
	  };
	};
      }else if ((scan>=3)&&(inputline.substr(0,3)=="MG ")) {
	int id;
	float tx,ty;
	sscanf(inputline.substr(3,scan-3).c_str(),"%d %f %f",&id,&tx,&ty);
	if (gsc->all.find(id)==gsc->all.end()) {
	  outputbuffer+="MM unexisting id in MG command\n";
	}else if (myid!=gsc->all[id]->get_int(cyborg::OWNERID)) {
	  outputbuffer+="MM am not owner\n";
	}else{
	  gsc->all[id]->get_int(cyborg::STATE)=cyborg::GOTO;
	  gsc->all[id]->get_float(cyborg::TARGETX)=tx;
	  gsc->all[id]->get_float(cyborg::TARGETY)=ty;
	};
      }else if ((scan>=3)&&(inputline.substr(0,3)=="MF ")) {
	int id,targetid;
	sscanf(inputline.substr(3,scan-3).c_str(),"%d %d",&id,&targetid);
	if (gsc->all.find(id)==gsc->all.end()) {
	  outputbuffer+="MM unexisting id in MF command\n";
	}else if (gsc->all.find(targetid)==gsc->all.end()) {
	  outputbuffer+="MM unexisting targetid in MF command\n";
	}else if (myid!=gsc->all[id]->get_int(cyborg::OWNERID)) {
	  outputbuffer+="MM am not owner\n";
	}else{
	  gsc->all[id]->get_int(cyborg::STATE)=cyborg::FIGHT;
	  gsc->all[id]->get_int(cyborg::TARGETID)=targetid;
	};
      }else if ((scan>=3)&&(inputline.substr(0,3)=="MI ")) {
	int id;
	sscanf(inputline.substr(3,scan-3).c_str(),"%d",&id);
	if (gsc->all.find(id)==gsc->all.end()) {
	  outputbuffer+="MM unexisting id in MI command\n";
	}else if (myid!=gsc->all[id]->get_int(cyborg::OWNERID)) {
	  outputbuffer+="MM am not owner\n";
	}else{
	  gsc->all[id]->get_int(cyborg::STATE)=cyborg::IDLE;
	};
      }else if (scan) {
	outputbuffer+="MM unknown command. try 'help'\n";
      };

      inputline.erase(0,scan+1);
      scan=0;
    }else{
      scan++;
    };
  };

  if (gotaQN) QN();

  return true;
};

bool conn::manageoutput ()
{
  int retval;
  while (outputbuffer.size()) {
    retval=write(handle,outputbuffer.c_str(),outputbuffer.size()>1024 ? 1024 : outputbuffer.size());
    if (retval==-1) {
      if (errno==EAGAIN) {
	break;
      }else{
	return false;
      };
    }else if (retval==0) {
      return false;
    }else{
      stat_sentbytes+=retval;
      outputbuffer.erase(0,retval);
    };
  };
  return true;
};

void conn::displaystatistic()
{
  printf("ownerid %d   sent %d Bytes   received %d Bytes\n",
	 id(), stat_sentbytes, stat_recvbytes);
};

void conn::QN()
{
  for (int x=0;x<visimap.xsize();x++)
    for (int y=0;y<visimap.ysize();y++)
      visimap.get(x,y)=false; // invisible
  for (map<const int,cyborg *const>::iterator it=gsc->all.begin();it!=gsc->all.end();it++) {
    if ((it->second->get_int(cyborg::OWNERID)!=myid)&&(!bossmode)) continue;
    viewvisimapper->reset(visimapper::coordinate(int(it->second->get_float(cyborg::POSX)),
						 int(it->second->get_float(cyborg::POSY))));
    visimapper::coordinate dot;
    while (viewvisimapper->getnextdot(dot)) {
      if ((dot.x>=0)&&(dot.y>=0)&&(dot.x<visimap.xsize())&&(dot.y<visimap.ysize()))
	visimap.get(dot.x,dot.y) = true;
    };
  };
  
  outputbuffer+="RM";
  for (int x=0;x<gsf->xsize();x++) {
    for (int y=0;y<gsf->ysize();y++) {
      if (visimap.get(x,y)) {
	// visible
	for (int i=0;i<feld::floatvalue_count();i++) {
	  if (int(gsf->get(x,y).get_float(feld::floatvalue_type(i))*10.0)!=
	      int(oldfeld.get(x,y).get_float(feld::floatvalue_type(i))*10.0)) {
	    oldfeld.get(x,y)=gsf->get(x,y);
	    sprintf(clip," %d %d ",x,y);
	    outputbuffer += clip;
	    oldfeld.get(x,y).dump(clip);
	    outputbuffer += clip;
	    goto weiter;
	  };
	};
      weiter:;
      }else{
	// invisible
	if (oldfeld.get(x,y).get_float(feld::WATER)!=-1.0) {
	  oldfeld.get(x,y).get_float(feld::WATER)=-1.0;
	  sprintf(clip," %d %d x",x,y);
	  outputbuffer += clip;
	};
      };
    };
  };
  outputbuffer += '\n';
  
  outputbuffer += "RC";
  for (map<const int,cyborg *const>::iterator it=gsc->all.begin();it!=gsc->all.end();it++) {
    if (oldfeld.get(int(it->second->get_float(cyborg::POSX)),
		    int(it->second->get_float(cyborg::POSY))).
	get_float(feld::WATER)==-1.0) continue;
    
    outputbuffer += ' ';
    it->second->dump(clip);
    outputbuffer += clip;
  };
  outputbuffer += '\n';
};


connections::connections()
{
};

connections::~connections()
{
  for (map<const int, conn *const>::iterator i=all.begin();i!=all.end();i++)
    delete i->second;
};

void connections::add (conn *const newconn)
{
  all.insert (pair<const int, conn *const>(newconn->id(),newconn));
};

void connections::del (const int id)
{
  delete all[id];
  all.erase (id);
};

connections *gcon = 0;
