/*
  cadaverserver
  Copyright (C) 1999  Erik Thiele

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __CYBORG_HPP
#define __CYBORG_HPP

#include <map>

class cyborg {
public:
  cyborg ();

  enum boolvalue_type { ISMALE=0 };
  enum intvalue_type { STATE=0, TAKEDROP=1, TARGETID=2, ID=3, OWNERID=4 };
  enum floatvalue_type { FERTILITY=0, HEALTH=1, AGE=2, FIGHTSTRENGTH=3,
			 TARGETX=4, TARGETY=5, POSX=6, POSY=7,
			 WATER=8, GRAIN=9, GRASS=10, CARCASS=11, HAY=12, MEAT=13, BREAD=14, FOREST=15 };
  enum state_type { IDLE=0, GOTO=1, FIGHT=2, TAKE=3, DROP=4, EAT=5, CONVERT=6, BURN=7 };
  enum takedrop_type { TWATER=0, TGRAIN=1, TGRASS=2, TCARCASS=3, THAY=4, TMEAT=5, TBREAD=6, TFOREST=7 };

  inline static int boolvalue_count () { return 1; };
  inline static int intvalue_count () { return 5; };
  inline static int floatvalue_count () { return 16; };
  inline static int state_count () { return 8; };
  inline static int takedrop_count () { return 8; };

  inline bool& get_bool (const boolvalue_type type) { return boolvalue[type]; };
  inline int& get_int (const intvalue_type type) { return intvalue[type]; };
  inline float& get_float (const floatvalue_type type) { return floatvalue[type]; };

  static const char *const boolvalue_textdescription (const boolvalue_type number);
  static const char *const intvalue_textdescription (const intvalue_type number);
  static const char *const floatvalue_textdescription (const floatvalue_type number);
  static const char *const state_textdescription (const state_type number);
  static const char *const takedrop_textdescription (const takedrop_type number);

  void dump (char *buf) const;
  void load (const char *buf);

private:
  bool boolvalue[1];
  int intvalue[5];
  float floatvalue[16];
};

class spielcyborg {
public:
  spielcyborg ();
  spielcyborg (const spielcyborg &);
  spielcyborg & operator= (const spielcyborg &);
  ~spielcyborg ();

  void add (cyborg *const newcyborg); // i now own the cyborg object
  void del (const int id);
  void clear ();

  // you get direct access to the map but please do not use it.
  map<const int, cyborg *const> all;
};

#endif
