/*
  cadaverserver
  Copyright (C) 1999  Erik Thiele

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "keyboard.hpp"

#include "myspielfeld.hpp"
#include "mycyborg.hpp"
#include "conn.hpp"
#include "misc.hpp"
#include "fatal.hpp"

#include <unistd.h>
#include <stdio.h>
#include <errno.h>

void keyboardparser::help()
{
  printf(
	 "\n"
	 "\n"
	 "help          - give this help text\n"
	 "quit          - quit server and terminate game\n"
	 "ra            - randomize playfield\n"
	 "gr            - generate grain everywhere\n"
	 "ki id         - kill cyborg with id [id]\n"
	 "bo ownerid m  - put boss mode of [ownerid] to [m] (0=notboss 1=boss)\n"
	 "as id ownerid - assign cyborg [id] to [ownerid].\n"
	 "mk c o x y    - create count=[c] cyborgs owner=[o] at ([x],[y])\n"
	 "stat          - display connection statistics\n"
	 "\n"
	 "\n");
};

bool keyboardparser::parse()
{
  char c;
  int retval;
  while ((retval=read(0,&c,1))==1) {
    if (c<32) {
      if (inputline=="help") {
	help();
      }else if (inputline=="quit") {
	return false;
      }else if ((inputline.size()>=2)&&(inputline.substr(0,2)=="ra")) {
	randomize_spielfeld();
      }else if ((inputline.size()>=2)&&(inputline.substr(0,2)=="gr")) {
	for (int i=0;i<gsf->xsize();i++)for (int j=0;j<gsf->ysize();j++)
	  gsf->get(i,j).get_float(feld::GRAIN)=1.0;
      }else if ((inputline.size()>=2)&&(inputline.substr(0,2)=="ki")) {
	inputline.erase(0,2);
	int id;
	if (sscanf(inputline.c_str(),"%d",&id)!=1) {
	  fprintf(stderr,"wrong parameters\n");
	}else if (gsc->all.find(id)==gsc->all.end()) {
	  fprintf(stderr,"no such id\n");
	}else{
	  cyborg &c = *gsc->all[id];
	  gsf->get(c.get_float(cyborg::POSX),c.get_float(cyborg::POSY)).
	    get_float(feld::CARCASS) = 1.0;
	  gsc->del(id);
	};
      }else if ((inputline.size()>=2)&&(inputline.substr(0,2)=="bo")) {
	inputline.erase(0,2);
	int id,yesno;
	if (sscanf(inputline.c_str(),"%d %d",&id,&yesno)!=2) {
	  fprintf(stderr,"wrong parameters\n");
	}else if (gcon->all.find(id)==gcon->all.end()) {
	  fprintf(stderr,"no such id\n");
	}else{
	  gcon->all[id]->bossmode=yesno ? true : false;
	};
      }else if ((inputline.size()>=2)&&(inputline.substr(0,2)=="as")) {
	inputline.erase(0,2);
	int id,ownerid;
	if (sscanf(inputline.c_str(),"%d %d",&id,&ownerid)!=2) {
	  fprintf(stderr,"wrong parameters\n");
	}else if (gsc->all.find(id)==gsc->all.end()) {
	  fprintf(stderr,"no such id\n");
	}else if ((ownerid)&&(gcon->all.find(ownerid)==gcon->all.end())) {
	  fprintf(stderr,"no such owner\n");
	}else{
	  gsc->all[id]->get_int(cyborg::OWNERID)=ownerid;
	};
      }else if ((inputline.size()>=2)&&(inputline.substr(0,2)=="mk")) {
	inputline.erase(0,2);
	int count,ownerid,cx,cy;
	if (sscanf(inputline.c_str(),"%d %d %d %d",&count,&ownerid,&cx,&cy)!=4) {
	  fprintf(stderr,"wrong parameters\n");
	}else if ((ownerid)&&(gcon->all.find(ownerid)==gcon->all.end())) {
	  fprintf(stderr,"no such owner\n");
	}else if ((cx>=gsf->xsize())||(cy>=gsf->ysize())||(cx<0)||(cy<0)) {
	  fprintf(stderr,"position out of range\n");
	}else{
	  for (int i=0;i<count;i++) {
	    static bool alt=false;
	    cyborg *x=new cyborg;
	    x->get_int(cyborg::ID)=getid();
	    x->get_int(cyborg::OWNERID)=ownerid;
	    x->get_float(cyborg::POSX)=float(cx)+0.5;
	    x->get_float(cyborg::POSY)=float(cy)+0.5;
	    x->get_float(cyborg::HEALTH)=1.0;
	    x->get_float(cyborg::MEAT)=5.0;
	    x->get_bool(cyborg::ISMALE)=alt;
	    alt=!alt;
	    gsc->add(x);
	  };
	};
      }else if ((inputline.size()>=4)&&(inputline.substr(0,4)=="stat")) {
	inputline.erase(0,4);
	for (map<const int, conn *const>::iterator i=gcon->all.begin();i!=gcon->all.end();++i)
	  i->second->displaystatistic();
      }else if (inputline.size()) {
	fprintf(stderr,"unknown command (try 'help')\n");
      };
      printf("> "); fflush(stdout);
      inputline="";
    }else{
      inputline+=c;
    };
  };
  if (retval==0) fatal("EOF on stdin.");
  else if (errno!=EAGAIN) fatal_errno("read error on stdin");
  return true;
};
