/*
  cadaverserver
  Copyright (C) 1999  Erik Thiele

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "mycyborg.hpp"

#include "myspielfeld.hpp"
#include "misc.hpp"
#include "conn.hpp"

#include <math.h>
#include <stdio.h>

static bool calculate_thiscyborg (cyborg &c, float ti)
{
  c.get_float(cyborg::AGE)+=ti;
  if (c.get_float(cyborg::AGE)>1200.0) {
    // death of a cyborg !
    return false;
  };

  c.get_float(cyborg::HEALTH)-=ti/260;
  if (c.get_float(cyborg::HEALTH)<0.0) {
    // death of a cyborg !
    return false;
  };

  if (c.get_float(cyborg::FERTILITY)>0.2) {
    if (c.get_bool(cyborg::ISMALE)) {
      c.get_float(cyborg::FERTILITY)-=ti/100.0;
    }else{
      c.get_float(cyborg::FERTILITY)-=ti/300.0;
    };
    if (c.get_float(cyborg::FERTILITY)<=0.2) {
      c.get_float(cyborg::FERTILITY)=0.0;
      if (c.get_bool(cyborg::ISMALE)==false) {
	// birth of a new cyborg !
	if (c.get_int(cyborg::OWNERID)) {
	  // only if cyborg has owner there will be child
	  // otherwise sex-rotation doesn't work
	  cyborg *x=new cyborg;
	  x->get_int(cyborg::ID)=getid();
	  x->get_int(cyborg::OWNERID)=c.get_int(cyborg::OWNERID);
	  x->get_float(cyborg::POSX)=c.get_float(cyborg::POSX);
	  x->get_float(cyborg::POSY)=c.get_float(cyborg::POSY);
	  x->get_float(cyborg::HEALTH)=1.0;
	  
	  x->get_bool(cyborg::ISMALE)=gcon->all[c.get_int(cyborg::OWNERID)]->lastsex;
	  gcon->all[c.get_int(cyborg::OWNERID)]->lastsex=
	    !gcon->all[c.get_int(cyborg::OWNERID)]->lastsex;
	  gsc->add(x);
	};
      };
    };
  };

  switch (c.get_int(cyborg::STATE)) {
  case cyborg::IDLE: break;
  case cyborg::GOTO: {
    const float walkspeed = 0.5; // fields (not dots!) per second
    const float xd = c.get_float(cyborg::TARGETX)-c.get_float(cyborg::POSX);
    const float yd = c.get_float(cyborg::TARGETY)-c.get_float(cyborg::POSY);
    const float dw=hypotf(xd,yd) / walkspeed;

    if (ti>=dw) {
      c.get_int(cyborg::STATE)=cyborg::IDLE;
      c.get_float(cyborg::POSX)=c.get_float(cyborg::TARGETX);
      c.get_float(cyborg::POSY)=c.get_float(cyborg::TARGETY);
    }else{
      const float sd=ti/dw;
      c.get_float(cyborg::POSX)+=xd*sd;
      c.get_float(cyborg::POSY)+=yd*sd;
    };

    float max=gsf->xsize()-0.2;
    if (c.get_float(cyborg::POSX)>max) c.get_float(cyborg::POSX)=max;
    if (c.get_float(cyborg::POSX)<0.2) c.get_float(cyborg::POSX)=0.2;
    max=gsf->ysize()-0.2;
    if (c.get_float(cyborg::POSY)>max) c.get_float(cyborg::POSY)=max;
    if (c.get_float(cyborg::POSY)<0.2) c.get_float(cyborg::POSY)=0.2;
    break;
  };
  case cyborg::FIGHT: {
    if (gsc->all.find(c.get_int(cyborg::TARGETID))==gsc->all.end()) {
      c.get_int(cyborg::STATE)=cyborg::IDLE;
    }else{
      float myx=c.get_float(cyborg::POSX);
      float myy=c.get_float(cyborg::POSY);
      float itx=gsc->all[c.get_int(cyborg::TARGETID)]->get_float(cyborg::POSX);
      float ity=gsc->all[c.get_int(cyborg::TARGETID)]->get_float(cyborg::POSY);
      if (hypotf(myx-itx,myy-ity)>2.0) {
	c.get_int(cyborg::STATE)=cyborg::IDLE;
      }else{
	float avail=gsc->all[c.get_int(cyborg::TARGETID)]->get_float(cyborg::HEALTH);
	float subtract=c.get_float(cyborg::FIGHTSTRENGTH)*ti*0.01;
	
	avail-=subtract;
	if (avail<0.0) avail=0.0;
	gsc->all[c.get_int(cyborg::TARGETID)]->get_float(cyborg::HEALTH)=avail;
	c.get_float(cyborg::FIGHTSTRENGTH)+=ti*0.05;
	if (c.get_float(cyborg::FIGHTSTRENGTH)>=1.5)
	  c.get_float(cyborg::FIGHTSTRENGTH)=1.5;
      };
    };
    break;
  };
  case cyborg::TAKE: {
    float &onfeld = gsf->get(c.get_float(cyborg::POSX),c.get_float(cyborg::POSY)).
      get_float(feld::floatvalue_type(c.get_int(cyborg::TAKEDROP)+feld::WATER));
    float todo = ti*0.1;
    float &i_have = c.get_float(cyborg::floatvalue_type(c.get_int(cyborg::TAKEDROP)+cyborg::WATER));

    if (todo > onfeld) { // nothing left on field
      todo = onfeld;
      c.get_int(cyborg::STATE)=cyborg::IDLE;
    };

    if (todo >= 10.0 - i_have) { // can't carry that much
      todo = 10.0 - i_have;
      c.get_int(cyborg::STATE)=cyborg::IDLE;
    };

    i_have += todo;
    onfeld -= todo;
    break;
  };
  case cyborg::DROP: {
    float &onfeld = gsf->get(c.get_float(cyborg::POSX),c.get_float(cyborg::POSY)).
      get_float(feld::floatvalue_type(c.get_int(cyborg::TAKEDROP)+feld::WATER));
    float todo = ti*0.1;
    float &i_have = c.get_float(cyborg::floatvalue_type(c.get_int(cyborg::TAKEDROP)+cyborg::WATER));

    if (todo >= 1.0 - onfeld) { // no space left on field
      todo = 1.0 - onfeld;
      c.get_int(cyborg::STATE)=cyborg::IDLE;
    };

    if (todo > i_have) { // i have nothing left to drop
      todo = i_have;
      c.get_int(cyborg::STATE)=cyborg::IDLE;
    };

    i_have -= todo;
    onfeld += todo;
    break;
  };
  case cyborg::EAT: {
    if (c.get_int(cyborg::TAKEDROP)==cyborg::TBREAD) {
      float avail=c.get_float(cyborg::BREAD);
      float amount=ti*0.05;
      if (amount>avail) {
	amount=avail;
	c.get_int(cyborg::STATE)=cyborg::IDLE;
      };
      c.get_float(cyborg::HEALTH)+=amount;
      c.get_float(cyborg::BREAD)-=amount;
    }else if (c.get_int(cyborg::TAKEDROP)==cyborg::TMEAT) {
      float avail=c.get_float(cyborg::MEAT);
      float amount=ti*0.025;
      if (amount>avail) {
	amount=avail;
	c.get_int(cyborg::STATE)=cyborg::IDLE;
      };
      c.get_float(cyborg::HEALTH)+=amount*2.0;
      c.get_float(cyborg::MEAT)-=amount;
    }else{
      c.get_int(cyborg::STATE)=cyborg::IDLE;
    };
    break;
  };
  case cyborg::CONVERT: {
    if (c.get_int(cyborg::TAKEDROP)==cyborg::TBREAD) {
      float avail=c.get_float(cyborg::GRAIN);
      float amount=ti*0.02;
      if (amount>avail) {
	amount=avail;
	c.get_int(cyborg::STATE)=cyborg::IDLE;
      };
      c.get_float(cyborg::BREAD)+=amount*2.0;
      c.get_float(cyborg::GRAIN)-=amount;
      if (c.get_float(cyborg::BREAD)>10.0) { // this wastes a little bit of GRAIN
	c.get_float(cyborg::BREAD)=10.0;
	c.get_int(cyborg::STATE)=cyborg::IDLE;
      };
    }else if (c.get_int(cyborg::TAKEDROP)==cyborg::TMEAT) {
      float avail=c.get_float(cyborg::CARCASS);
      float amount=ti*0.01;
      if (amount>avail) {
	amount=avail;
	c.get_int(cyborg::STATE)=cyborg::IDLE;
      };
      c.get_float(cyborg::MEAT)+=amount*0.5;
      c.get_float(cyborg::CARCASS)-=amount;
      if (c.get_float(cyborg::MEAT)>10.0) { // this wastes a little bit of CARCASS
	c.get_float(cyborg::MEAT)=10.0;
	c.get_int(cyborg::STATE)=cyborg::IDLE;
      };
    }else{
      c.get_int(cyborg::STATE)=cyborg::IDLE;
    };
    break;
  };
  case cyborg::BURN: {
    float todo = ti*0.05;
    float &i_have = c.get_float(cyborg::floatvalue_type(c.get_int(cyborg::TAKEDROP)+cyborg::WATER));

    if (todo > i_have) { // i have nothing left to burn
      todo = i_have;
      c.get_int(cyborg::STATE)=cyborg::IDLE;
    };

    i_have -= todo;
    break;
  };
  };

  return true; // i survived.
};

void calculate_spielcyborg (float ti)
{
  for (map<const int, cyborg *const>::iterator i=gsc->all.begin();i!=gsc->all.end();) {
    cyborg &c = *((i++)->second);
    const int id=c.get_int(cyborg::ID);
    if (!calculate_thiscyborg(c,ti)) {
      gsf->get(c.get_float(cyborg::POSX),c.get_float(cyborg::POSY)).
	get_float(feld::CARCASS) = 1.0;
      gsc->del(id);
    };
  };
};


spielcyborg *gsc=0;
