/*
  cadaverserver
  Copyright (C) 1999  Erik Thiele

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "myspielfeld.hpp"

#include <stdlib.h>
#include <math.h>

static inline void calculate_thisfeld (feld &f, const float ti)
{
  const float ts=0.005; //totalscale

  f.get_float(feld::FOREST) += ti*1.0*ts  *  (0.1*exp(f.get_float(feld::FOREST)*2.3));
  f.get_float(feld::GRAIN) += ti*1.0*ts  *  (0.1*exp(f.get_float(feld::GRAIN)*2.3));
  f.get_float(feld::GRASS) += ti*2.0*ts  *  (0.1*exp(f.get_float(feld::GRASS)*2.3));
  f.get_float(feld::CARCASS) -= ti*1.0*ts  *  (0.1*exp(f.get_float(feld::CARCASS)*2.3));

  f.get_float(feld::GRAIN) -= 2  *  ti*0.5*ts  *  (0.1*exp(f.get_float(feld::FOREST)*2.3)*f.get_float(feld::FOREST));
  f.get_float(feld::GRASS) -= 1.1  *  ti*2.0*ts  *  (0.1*exp(f.get_float(feld::FOREST)*2.3)*f.get_float(feld::FOREST));

  f.get_float(feld::GRAIN) -= 1.5  *  ti*0.5*ts  *  (0.1*exp(f.get_float(feld::GRASS)*2.3)*f.get_float(feld::GRASS));

  if (f.get_float(feld::FOREST)>1.0)f.get_float(feld::FOREST)=1.0;
  if (f.get_float(feld::FOREST)<0.0)f.get_float(feld::FOREST)=0.0;
  if (f.get_float(feld::GRAIN)>1.0)f.get_float(feld::GRAIN)=1.0;
  if (f.get_float(feld::GRAIN)<0.0)f.get_float(feld::GRAIN)=0.0;
  if (f.get_float(feld::GRASS)>1.0)f.get_float(feld::GRASS)=1.0;
  if (f.get_float(feld::GRASS)<0.0)f.get_float(feld::GRASS)=0.0;
  if (f.get_float(feld::CARCASS)>1.0)f.get_float(feld::CARCASS)=1.0;
  if (f.get_float(feld::CARCASS)<0.0)f.get_float(feld::CARCASS)=0.0;
};

void calculate_spielfeld (float ti)
{
  for (int x=0;x<gsf->xsize();x++)
    for (int y=0;y<gsf->ysize();y++)
      calculate_thisfeld(gsf->get(x,y),ti);
};

static void recurs(int x1,int y1,  int x2, int y2, feld::floatvalue_type value, float level,float vibra)
{
  int x=(x1+x2)>>1;
  int y=(y1+y2)>>1;

  if (gsf->get(x,y).get_float(value)>-1.0) return;
  float flo=gsf->get(x1,y1).get_float(value);
  float fro=gsf->get(x2,y1).get_float(value);
  float flu=gsf->get(x1,y2).get_float(value);
  float fru=gsf->get(x2,y2).get_float(value);

  float fmo=gsf->get(x,y1).get_float(value);
  float fmu=gsf->get(x,y2).get_float(value);
  float fml=gsf->get(x1,y).get_float(value);
  float fmr=gsf->get(x2,y).get_float(value);

  if (fmo<-1.0) {
    fmo=((rand()%1001-500)/vibra)*level+(flo+fro)/2.0;
    if (fmo>1.0)fmo=1.0; if (fmo<0.0) fmo=0.0;
    gsf->get(x,y1).get_float(value)=fmo;
  };

  if (fmu<-1.0) {
    fmu=((rand()%1001-500)/vibra)*level+(flu+fru)/2.0;
    if (fmu>1.0)fmu=1.0; if (fmu<0.0) fmu=0.0;
    gsf->get(x,y2).get_float(value)=fmu;
  };

  if (fml<-1.0) {
    fml=((rand()%1001-500)/vibra)*level+(flo+flu)/2.0;
    if (fml>1.0)fml=1.0; if (fml<0.0) fml=0.0;
    gsf->get(x1,y).get_float(value)=fml;
  };

  if (fmr<-1.0) {
    fmr=((rand()%1001-500)/vibra)*level+(fro+fru)/2.0;
    if (fmr>1.0)fmr=1.0; if (fmr<0.0) fmr=0.0;
    gsf->get(x2,y).get_float(value)=fmr;
  };

  gsf->get(x,y).get_float(value)=(fmo+fmu+fml+fmr)/4.0;

  recurs(x1,y1,x ,y ,value,level+1,vibra);
  recurs(x ,y1,x2,y ,value,level+1,vibra);
  recurs(x1,y ,x ,y2,value,level+1,vibra);
  recurs(x ,y ,x2,y2,value,level+1,vibra);
};

static void randomize_withvalue (feld::floatvalue_type value,float edge,float vibrafac)
{
  for (int x=0;x<gsf->xsize();x++)
    for (int y=0;y<gsf->ysize();y++)
      gsf->get(x,y).get_float(value)=-2.0;
  gsf->get(0              , 0              ).get_float(value)=edge;
  gsf->get(0              , gsf->ysize()-1 ).get_float(value)=edge;
  gsf->get(gsf->xsize()-1 , 0              ).get_float(value)=edge;
  gsf->get(gsf->xsize()-1 , gsf->ysize()-1 ).get_float(value)=edge;
  float vibra = gsf->xsize() > gsf->ysize() ? gsf->xsize() : gsf->ysize();
  vibra=log(vibra)/log(2.0)*vibrafac;
  recurs(0,0,gsf->xsize()-1,gsf->ysize()-1,value,1,vibra);
};

void putdots(feld::floatvalue_type value, float percentage, float putter)
{
  int count = int((gsf->xsize()*gsf->ysize()) * percentage);

  for (int i=0;i<count;i++)
    gsf->get(rand()%gsf->xsize(),rand()%gsf->ysize()).get_float(value)=putter;
};

void randomize_spielfeld()
{
  randomize_withvalue(feld::WATER,0.5,2000.0);
  randomize_withvalue(feld::GRAIN,0.2,3000.0);
  randomize_withvalue(feld::GRASS,0.2,2000.0);
  randomize_withvalue(feld::FOREST,0.2,3000.0);
  putdots(feld::CARCASS,0.01,1.0);
  putdots(feld::HAY,0.01,0.2);
  putdots(feld::MEAT,0.01,0.2);
  putdots(feld::BREAD,0.01,0.2);
};


spielfeld *gsf=0;
