/*
  panzer
  Copyright (C) 1999  Erik Thiele

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "selector.hpp"

#include "fatal.hpp"

#include <sys/time.h>

selector::selector ()
{
  reset();
};

void selector::reset()
{
  FD_ZERO(&readfds);
  FD_ZERO(&writefds);
  infinity=true;
  timeout_expired=false;
};

void selector::add_read(const int handle)
{
  FD_SET(handle,&readfds);
};

void selector::add_write(const int handle)
{
  FD_SET(handle,&writefds);
};

void selector::settimeout(const zeit &timeout_)
{
  infinity=false;
  timeout=timeout_;
};

void selector::waitnow ()
{
  struct timeval tv;
  timeout.convert_to_timeval(tv);
  int retval = select (FD_SETSIZE,&readfds,&writefds,0,infinity ? 0 : &tv);
  if (retval < 0) fatal_errno("selector::waitnow(): select returned error");
  timeout_expired = (retval==0);
};

bool selector::isready_read(const int handle)
{
  return FD_ISSET(handle,&readfds);
};

bool selector::isready_write(const int handle)
{
  return FD_ISSET(handle,&writefds);
};
