/*
  cadaverserver
  Copyright (C) 1999  Erik Thiele

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "visimapper.hpp"

#include <map>
#include <math.h>

struct orderedcoordinate : public visimapper::coordinate {
  inline orderedcoordinate (const int x_, const int y_)
    : coordinate(x_,y_) {};

  inline bool operator< (const orderedcoordinate &other) const {
    if (x<other.x)
      return true;
    else if (x==other.x)
      return y<other.y;
    else // x>other.x
      return false;
  };
};

static void visit (const orderedcoordinate &c, const float rad, map<orderedcoordinate,bool> &m)
{
  if (m.find(c)!=m.end()) return; // already visited ?
  float mydistance = hypotf(c.x, c.y);
  if (mydistance > rad) return; // too far away ?
  m[c] = true;

  visit (orderedcoordinate(c.x+1,c.y),rad,m);
  visit (orderedcoordinate(c.x-1,c.y),rad,m);
  visit (orderedcoordinate(c.x,c.y+1),rad,m);
  visit (orderedcoordinate(c.x,c.y-1),rad,m);

  visit (orderedcoordinate(c.x+1,c.y+1),rad,m);
  visit (orderedcoordinate(c.x-1,c.y-1),rad,m);
  visit (orderedcoordinate(c.x+1,c.y-1),rad,m);
  visit (orderedcoordinate(c.x-1,c.y+1),rad,m);
};

visimapper::visimapper(const float radius)
{
  map<orderedcoordinate,bool> usedmap;
  visit (orderedcoordinate(0,0),radius,usedmap);
  map<orderedcoordinate,bool>::const_iterator i;
  for (i=usedmap.begin();i!=usedmap.end();++i)
    vec.push_back(i->first);
  end = vec.end();
};
