/*
  cadaverserver
  Copyright (C) 1999  Erik Thiele

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __VISIMAPPER_HPP
#define __VISIMAPPER_HPP

#include <vector>

/*

  visimapper is used to determine which fields are visible by a cyborg.
  you give the midpoint and the viewing radius.
  
  the constructor is SLOW ! and thus should be used only ONCE
  this is possible, because the radius is the same all time.
  
  if you want to get the dots, call reset with your midpoint,
  and then getnextdot until it returns false.

  reset and getnextdot are FAST.

*/

class visimapper {
public:
  struct coordinate {
    inline coordinate () {}; // leave undefined
    inline coordinate (const int x_, const int y_) : x(x_), y(y_) {};
    int x,y;
  };
  
  visimapper(const float radius);

  inline void reset(const coordinate &mid_) {
    mid=mid_;
    cur=vec.begin();
  };

  inline bool getnextdot(coordinate &c) {
    if (cur==end) return false;
    c = *(cur++);
    c.x += mid.x;
    c.y += mid.y;
    return true;
  };

private:
  typedef vector <coordinate> vec_type;
  vec_type vec;
  vec_type::iterator cur, end;
  coordinate mid;
};

#endif
