/*
  panzer
  Copyright (C) 1999  Erik Thiele

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "zeit.hpp"

#include <unistd.h>
#include <sys/time.h>

zeit::zeit()
{
  s = us = 0;
};

zeit::zeit (const int seconds)
{
  s = seconds;
  us = 0;
};

zeit::zeit (const int seconds, const int microseconds)
{
  s = seconds;
  us = microseconds;
  normalize();
};

void zeit::getcurrenttime()
{
  timeval tv;
  gettimeofday (&tv, 0);
  s = tv.tv_sec;
  us = tv.tv_usec;
};

void zeit::add (const zeit &other)
{
  s += other.s;
  us += other.us;
  normalize ();
};

void zeit::subtract (const zeit &other)
{
  s -= other.s;
  us -= other.us;
  normalize ();
};

void zeit::normalize ()
{
  while (us<0) {
    us+=1000000;
    s--;
  };
  while (us>=1000000) {
    us-=1000000;
    s++;
  };
};

float zeit::tofloat() const
{
  float res = us;
  res /= 1000000;
  res += float (s);
  return res;
};

bool zeit::isnegative() const
{
  return s < 0;
};

void zeit::convert_to_timeval (struct timeval & tv) const
{
  tv.tv_sec = s;
  tv.tv_usec = us;
};


const bool operator< (const zeit &a, const zeit &b)
{
  if (a.s<b.s) {
    return true;
  }else if (a.s==b.s) {
    return a.us<b.us;
  }else{ // a.s>b.s
    return false;
  };
};
